/*
 * Argus Software.  Argus files - Input includes
 * Copyright (c) 2000-2011 QoSient, LLC
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * $Id: //depot/argus/argus/argus/ArgusSource.h#31 $
 * $DateTime: 2011/02/25 13:44:58 $
 * $Change: 2117 $
 */

/*  ArgusSource.h */

#ifndef ArgusSource_h
#define ArgusSource_h

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <string.h>
#include <strings.h>

#if defined(ARGUS_TILERA)
#include <argus_tilera.h>

#include <sys/archlib.h>
#include <pass.h>
#include <sys/tile_io.h>
#include <sys/netio.h>
#include <sys/event.h>
#include <ilib.h>
#include <stdio.h>
#include <sys/simulator.h>
#include <sys/profiler.h>

#else
#include <pcap.h>
#endif

/*-
 * Copyright (c) 2003, 2004 David Young.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of David Young may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY DAVID YOUNG ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DAVID
 * YOUNG BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */

#ifndef _CPACK_H
#define _CPACK_H

struct cpack_state {
	u_int8_t					*c_buf;
	u_int8_t					*c_next;
	size_t						 c_len;
};

int cpack_init(struct cpack_state *, u_int8_t *, size_t);

int cpack_uint8(struct cpack_state *, u_int8_t *);
int cpack_uint16(struct cpack_state *, u_int16_t *);
int cpack_uint32(struct cpack_state *, u_int32_t *);
int cpack_uint64(struct cpack_state *, u_int64_t *);

#define cpack_int8(__s, __p)	cpack_uint8((__s),  (u_int8_t*)(__p))
#define cpack_int16(__s, __p)	cpack_uint16((__s), (u_int16_t*)(__p))
#define cpack_int32(__s, __p)	cpack_uint32((__s), (u_int32_t*)(__p))
#define cpack_int64(__s, __p)	cpack_uint64((__s), (u_int64_t*)(__p))

#endif /* _CPACK_H */
 
int pcap_read(pcap_t *, int cnt, pcap_handler, u_char *);
int pcap_offline_read(pcap_t *, int, pcap_handler, u_char *);

#include <signal.h>
#include <errno.h>

#include <argus/extract.h>
#include <argus/fddi.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>

#if defined(__NetBSD__)
#include <netinet/in.h>
#endif

#if !defined(__OpenBSD__)
#include <netinet/if_ether.h>
#endif

#include <netinet/in.h>

#if !defined(__OpenBSD__) || (defined(__OpenBSD__) && !defined(_NETINET_IF_SYSTEM_H_))
#define _NETINET_IF_SYSTEM_H_
#include <netinet/in_systm.h>
#endif

#ifndef _NETINET_IP_H_
#include <netinet/ip.h>
#define _NETINET_IP_H_
#endif

#ifndef _NETINET_UDP_H_
#include <netinet/udp.h>
#define _NETINET_UDP_H_
#endif

#include <netinet/tcp.h>

 
#define ARGUSLIBPPKTFILE	1
#define ARGUSSNOOPKTFILE	2
#define ARGUSMOATTSHPKTFILE	3
#define ARGUSDAGLINK            4
#define ARGUSERFPKTFILE		5

#define ARGUS_FILE_DEVICE	1
#define ARGUS_LIVE_DEVICE	2


#define ARGUS_MAXINTERFACE	64

#define ARGUS_TYPE_IND		0x01
#define ARGUS_TYPE_BOND		0x02
#define ARGUS_TYPE_DUPLEX	0x04

struct ArgusDeviceStruct {
   struct ArgusListObjectStruct *nxt;
   int status, type, link, idtype, dlt;
   struct ArgusAddrStruct ArgusID;
   char *name, *dltname;
   struct ArgusListStruct *list;
#if defined(ARGUS_TILERA)
   netio_input_config_t config;
#endif
};

struct ArgusRfileStruct {
   struct ArgusListObjectStruct *nxt;
   char *name;
};

struct arguspcap_sf {
   FILE *rfile;
   int swapped;
   int hdrsize;
   int version_major;
   int version_minor;
   u_char *base;
};
  
struct arguspcap_md {
   struct pcap_stat stat;
   /*XXX*/
   int use_bpf;		/* using kernel filter */
   u_long	TotPkts;	/* can't oflow for 79 hrs on ether */
   u_long	TotAccepted;	/* count accepted by filter */
   u_long	TotDrops;	/* count of dropped packets */
   long	TotMissed;	/* missed by i/f during this run */
   long	OrigMissed;	/* missed by i/f before this run */
#ifdef linux
   int	sock_packet;	/* using Linux 2.0 compatible interface */
   int	timeout;	/* timeout specified to pcap_open_live */
   int	clear_promisc;	/* must clear promiscuous mode when we close */
   int	cooked;		/* using SOCK_DGRAM rather than SOCK_RAW */
   int	lo_ifindex;	/* interface index of the loopback device */
   char 	*device;	/* device name */
   struct pcap *next;	/* list of open promiscuous sock_packet pcaps */
#endif
};
  
struct arguspcap {
   int fd;
   int snapshot;
   int linktype;
   int tzoff;		/* timezone offset */
   int offset;		/* offset for proper alignment */
 
   struct arguspcap_sf sf;
   struct arguspcap_md md;
  
   /*
    * Read buffer.
    */
   int bufsize;
   u_char *buffer;
   u_char *bp;
   int cc;
  
   /*
    * Place holder for pcap_next().
    */
   u_char *pkt;
 
   /*
    * Placeholder for filter code if bpf not in kernel.
    */
   struct bpf_program fcode;
  
   char errbuf[PCAP_ERRBUF_SIZE];
};

#define MAXSTRSIZE 1024
#define ARGUS_MAXSNAPLEN 65535
 
struct ArgusInterfaceStruct {
   struct ArgusDeviceStruct *ArgusDevice;
   struct ifreq ifr;
   pcap_t *ArgusPd;
   struct bpf_program ArgusFilter;
   int state, index, ArgusInterfaceType;
   pcap_handler ArgusCallBack;

   struct arguspcap ArgusPcap;

   unsigned int ArgusLocalNet, ArgusNetMask;
   struct pcap_stat ArgusStat;

   long long ArgusTotalPkts;
   long long ArgusTotalIPPkts;
   long long ArgusTotalNonIPPkts;
   long long ArgusLastPkts;
   long long ArgusTotalDrop;
   long long ArgusLastDrop;
   long long ArgusTotalBytes;
   long long ArgusLastBytes;

   u_char *ArgusPacketBuffer;
   u_char ArgusPacketBufferBuffer[ARGUS_MAXSNAPLEN];
};


struct ArgusMoatTshPktHdr {
   unsigned int sec;
   char interface;
   char usec[3];
   struct ip ip;
   struct tcphdr tcp;
};

#define SNOOP_FILE_OFFSET 16

struct ArgusSnoopPktHdr {
    unsigned int len;
    unsigned int tlen;
    unsigned int pad[2];
    struct timeval argtvp;
};


#define ARGUS_HOLDING	1

struct ArgusSourceStruct {
   int state, status, proc;
 
   struct ArgusListStruct *ArgusDeviceList;
   struct ArgusListStruct *ArgusRfileList;
   struct ArgusModelerStruct *ArgusModel;
 
   char *ArgusInputFilter;

   struct ArgusAddrStruct ArgusID;
   int type;

   struct timeval ArgusStartTime, ArgusEndTime, lasttime;

   int ArgusSnapLength, ArgusThisLength;
   unsigned char *ArgusThisSnapEnd;

   struct ieee80211_radiotap ArgusThisRadioTap;

   unsigned char ArgusInterfaceType;
   unsigned char ArgusInterfaceStatus;
  
   int Argustflag, sNflag, eNflag, kflag, pflag, uflag, tflag;
   float Tflag;

   pcap_if_t *ArgusPacketDevices;
   pcap_if_t *ArgusDevice;

   int ArgusInterfaceIndex, ArgusThisIndex, ArgusInterfaces;
   struct ArgusInterfaceStruct ArgusInterface[ARGUS_MAXINTERFACE];
   struct ArgusSourceStruct *srcs[ARGUS_MAXINTERFACE];

#if defined(ARGUS_THREADS)
   pthread_t thread;
   pthread_mutex_t lock;
   pthread_cond_t cond;
#endif

#if defined(ARGUS_TILERA)
  netio_queue_t queue;
#endif

   int ArgusInputPacketFileType;
   int ArgusReadingOffLine;
   int Argusbpflag, ArgusCaptureFlag;
  
   FILE *ArgusPacketInput;
 
   int ArgusSnapLen, ArgusOflag, Arguspflag;
   char **ArgusArgv;
   int ArgusOptind;
   char *ArgusCmdBuf;

   char *ArgusWriteOutPacketFile;
   pcap_dumper_t *ArgusPcapOutFile;
};


void ArgusParseSourceID (struct ArgusSourceStruct *, char *);

int ArgusSnoopRead (struct ArgusSourceStruct *);

void ArgusIpPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusArcnetPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusEtherPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusTokenPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusAtmClipPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusLoopPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusHdlcPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusSlipPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusPppPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusPppBsdosPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusFddiPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusATMPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusSllPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusPppHdlcPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusPppEtherPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void Argus802_11Packet (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusLtalkPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusPrismPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void Argus802_11RadioPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void Argus802_11RadioAvsPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusEncPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusDagPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void ArgusNullPacket (u_char *user, const struct pcap_pkthdr *h, const u_char *p);


struct callback {
   pcap_handler function;
   int type;
   char *fname;
};

void ArgusSourceProcess (struct ArgusSourceStruct *);
void *ArgusGetPackets (void *);
struct ArgusDeviceStruct *ArgusCloneDevice(struct ArgusDeviceStruct *);


int ArgusOpenInputPacketFile(struct ArgusSourceStruct *, struct ArgusDeviceStruct *, struct ArgusInterfaceStruct *);

pcap_handler Arguslookup_pcap_callback (int);
pcap_handler Arguslookup_dag_callback (int);
void Argusbpf_dump(struct bpf_program *, int);

void setArgusRealTime (struct ArgusSourceStruct *, float value);
float getArgusRealTime (struct ArgusSourceStruct *);

unsigned int getArgusID(struct ArgusAddrStruct *);
void setArgusID(struct ArgusAddrStruct *, unsigned int);

void setArgusMoatTshFile (struct ArgusSourceStruct *, int value);
int getArgusMoatTshFile (struct ArgusSourceStruct *);

void setArgusWriteOutPacketFile (struct ArgusSourceStruct *, char *);



unsigned char getArgusInterfaceType(struct ArgusSourceStruct *);
void setArgusInterfaceType(struct ArgusSourceStruct *, unsigned char);
 
unsigned char getArgusInterfaceStatus(struct ArgusSourceStruct *);
void setArgusInterfaceStatus(struct ArgusSourceStruct *, unsigned char);


#if defined(ArgusSource)

static struct callback ArgusSourceCallbacks[] = {
#if defined(ARGUS_TILERA)
#else
   { ArgusArcnetPacket,    DLT_ARCNET,      "ArgusArcnetPacket()" },
   { ArgusEtherPacket,     DLT_EN10MB,      "ArgusEtherPacket()" },
   { ArgusTokenPacket,     DLT_IEEE802,     "ArgusTokenPacket()" },
   { ArgusSlipPacket,      DLT_SLIP,        "ArgusSlipPacket()" },
   { ArgusSlipPacket,      DLT_SLIP_BSDOS,  "ArgusSlipPacket()" },
   { ArgusPppPacket,       DLT_PPP,         "ArgusPppPacket()" },
   { ArgusPppBsdosPacket,  DLT_PPP_BSDOS,   "ArgusPppBsdosPacket()" },
   { ArgusFddiPacket,      DLT_FDDI,        "ArgusFddiPacket()" },
   { ArgusATMPacket,       DLT_ATM_RFC1483, "ArgusATMPacket()" },
   { ArgusIpPacket,        DLT_RAW,         "ArgusIpPacket()" },
   { ArgusNullPacket,      DLT_NULL,        "ArgusNullPacket()" },
#ifdef DLT_ENC
   { ArgusEncPacket,       DLT_ENC,         "ArgusEncPacket()" },
#endif
#ifdef DLT_LANE8023
   { ArgusLanePacket,      DLT_LANE8023,    "ArgusLanePacket()" },
#endif
#ifdef DLT_CIP
   { ArgusCipPacket,       DLT_CIP,         "ArgusCipPacket()" },
#endif
#ifdef DLT_ATM_CLIP
   { ArgusAtmClipPacket,   DLT_ATM_CLIP,    "ArgusAtmClipPacket()" },
#endif
#ifdef DLT_LINUX_SLL
   { ArgusSllPacket,       DLT_LINUX_SLL,   "ArgusSllPacket()" },
#endif
#ifdef DLT_LOOP
   { ArgusLoopPacket,      DLT_LOOP,        "ArgusLoopPacket()" },
#endif
#ifdef DLT_C_HDLC
   { ArgusHdlcPacket,      DLT_C_HDLC,      "ArgusHdlcPacket()" },
#endif
#ifdef DLT_HDLC
   { ArgusHdlcPacket,      DLT_HDLC,        "ArgusHdlcPacket()" },
#endif
#ifdef DLT_PPP_SERIAL
   { ArgusPppHdlcPacket,   DLT_PPP_SERIAL,  "ArgusPppHdlcPacket()" },
#endif
#ifdef DLT_PPP_ETHER
   { ArgusPppEtherPacket,  DLT_PPP_ETHER,   "ArgusPppEtherPacket()" },
#endif
#ifdef DLT_LINUX_SLL
   { ArgusSllPacket,       DLT_LINUX_SLL,   "ArgusSllPacket()" },
#endif
#ifdef DLT_IEEE802_11
   { Argus802_11Packet,    DLT_IEEE802_11,  "Argus802_11Packet()"},
#endif
#ifdef DLT_PRISM_HEADER
   { ArgusPrismPacket,     DLT_PRISM_HEADER, "ArgusPrismPacket()" },
#endif
#ifdef DLT_IEEE802_11_RADIO
   { Argus802_11RadioPacket, DLT_IEEE802_11_RADIO, "Argus802_11RadioPacket()" },
#endif
#ifdef DLT_IEEE802_11_RADIO_AVS
   { Argus802_11RadioAvsPacket, DLT_IEEE802_11_RADIO_AVS, "Argus802_11RadioAvsPacket()" },
#endif
#ifdef DLT_LTALK
   { ArgusLtalkPacket,     DLT_LTALK,       "ArgusLtalkPacket()" },
#endif
   { NULL, DLT_NULL, "" },
#endif
   { NULL, 0, NULL},
};

extern int Argustflag;

struct ArgusSourceStruct *ArgusSourceTask = NULL;

struct ArgusSourceStruct *ArgusNewSource(struct ArgusModelerStruct *);
struct ArgusSourceStruct *ArgusCloneSource(struct ArgusSourceStruct *);
int ArgusInitSource(struct ArgusSourceStruct *);
int ArgusCloseSource(struct ArgusSourceStruct *);

extern char *ArgusCopyArgv (char **argv);

void setArgusOutputTask(void);
void setArgusModeler(struct ArgusSourceStruct *);

 
struct ArgusOutputStruct *getArgusOutputTask(void);
struct ArgusModelerStruct *getArgusModeler(struct ArgusSourceStruct *);

int getArgusSnapLen(struct ArgusSourceStruct *);
void setArgusSnapLen(struct ArgusSourceStruct *, int);

int getArgusbpflag(struct ArgusSourceStruct *);
int getArguspflag(struct ArgusSourceStruct *);
int getArgusOflag(struct ArgusSourceStruct *);

void setArgusbpflag(struct ArgusSourceStruct *, int);
void setArguspflag(struct ArgusSourceStruct *, int);
void setArgusOflag(struct ArgusSourceStruct *, int);
void setArgusCaptureFlag(struct ArgusSourceStruct *, int);

void setArgusDevice(struct ArgusSourceStruct *, char *, int);
void setArgusInfile(struct ArgusSourceStruct *, char *);
void setArgusrfile(struct ArgusSourceStruct *, char *);
void setArgusrFile(struct ArgusSourceStruct *, char *);

char *getArgusDevice(struct ArgusSourceStruct *);
char *getArgusInfile(struct ArgusSourceStruct *);
char *getArgusrfile(struct ArgusSourceStruct *);

void clearArgusDevice(struct ArgusSourceStruct *);

int ArgusCreatePktFromFddi(const struct fddi_header *, struct ether_header *, int);


#else /* defined(ArgusSource) */

extern struct ArgusSourceStruct *ArgusSourceTask;

extern long long ArgusTotalPkts;
extern long long ArgusLastPkts;
extern long long ArgusTotalDrop;
extern long long ArgusLastDrop;
extern long long ArgusTotalBytes;
extern long long ArgusLastBytes;

extern struct ArgusSourceStruct *ArgusSourceTask;

extern struct ArgusSourceStruct *ArgusNewSource(struct ArgusModelerStruct *);
extern struct ArgusSourceStruct *ArgusCloneSource(struct ArgusSourceStruct *);
extern int ArgusInitSource(struct ArgusSourceStruct *);
extern int ArgusCloseSource(struct ArgusSourceStruct *);

extern struct ArgusOutputStruct *getArgusOutputTask(struct ArgusSourceStruct *);
extern void setArgusOutputTask(struct ArgusSourceStruct *);

extern struct ArgusModelerStruct *getArgusModeler(struct ArgusSourceStruct *);
extern void setArgusModeler(struct ArgusSourceStruct *);

extern int getArgusSnapLen(struct ArgusSourceStruct *);
extern void setArgusSnapLen(struct ArgusSourceStruct *, int);

extern int getArgusbpflag(struct ArgusSourceStruct *);
extern int getArguspflag(struct ArgusSourceStruct *);
extern int getArgusOflag(struct ArgusSourceStruct *);
extern int getArgusMoatTshFile (struct ArgusSourceStruct *);

extern void setArgusbpflag(struct ArgusSourceStruct *, int);
extern void setArguspflag(struct ArgusSourceStruct *, int);
extern void setArgusOflag(struct ArgusSourceStruct *, int);
extern void setArgusCaptureFlag(struct ArgusSourceStruct *, int);
extern void setArgusMoatTshFile (struct ArgusSourceStruct *, int value);

extern void setArgusWriteOutPacketFile (struct ArgusSourceStruct *, char *);

extern void setArgusDevice(struct ArgusSourceStruct *, char *, int);
extern void setArgusInfile(struct ArgusSourceStruct *, char *);
extern void setArgusrfile(struct ArgusSourceStruct *, char *);

extern char *getArgusDevice(struct ArgusSourceStruct *);
extern char *getArgusInfile(struct ArgusSourceStruct *);
extern char *getArgusrfile(struct ArgusSourceStruct *);

extern void clearArgusDevice(struct ArgusSourceStruct *);

#endif
#endif /* #ifndef ArgusSource_h */
